'''Views for Djournal.'''

from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext

from taggit.models import Tag

from djournal.models import Entry

def entry_index(request, limit=0, template='djournal/entry_index.html'):
    '''Returns a reponse of a fixed number of entries; all of them, by default. '''

    entries = Entry.public.all()

    if limit > 0:
        entries = entries[:limit]

    context = {
        'entries': entries,
    }

    return render_to_response(
        template,
        context,
        context_instance=RequestContext(request),
    )

def entry_detail(request, slug, template='djournal/entry_detail.html'):
    '''Returns a response of an individual entry, for the given slug.'''

    entry = get_object_or_404(Entry.public, slug=slug)

    context = {
        'entry': entry,
    }

    return render_to_response(
        template,
        context,
        context_instance=RequestContext(request),
    )

def tagged_entry_index(request, slug, template='djournal/tagged_entry_index.html'):
    '''Returns a response of all entries tagged with a given tag.'''

    tag = get_object_or_404(Tag, slug=slug)

    entries = Entry.public.filter(tags__in=[tag])

    context = {
        'entries': entries,
        'tag': tag,
    }

    return render_to_response(
        template,
        context,
        context_instance=RequestContext(request),
    )



'''Models for Djournal.'''

from django.contrib.auth.models import User
from django.db import models

from taggit.managers import TaggableManager

from djournal import managers

try:
    from ripwrap.core import render
except ImportError:
    render = lambda text: text

class Entry(models.Model):
    '''Represents a Djournal entry.'''

    title = models.CharField(max_length=128)
    subtitle = models.CharField(max_length=128, blank=True)

    slug = models.SlugField(max_length=128, unique=True)

    author = models.ForeignKey(User, related_name='entries')

    created = models.DateTimeField()
    modified = models.DateTimeField()

    tags = TaggableManager()

    teaser = models.TextField(blank=True)
    body = models.TextField(blank=True)

    teaser_html = models.TextField(blank=True, editable=False)
    body_html = models.TextField(blank=True, editable=False)

    PUBLIC_ENTRY_STATUS = 'public'
    PRIVATE_ENTRY_STATUS = 'private'
    DRAFT_ENTRY_STATUS = 'draft'

    ENTRY_STATUSES = (
        (PUBLIC_ENTRY_STATUS, 'Public'),
        (PRIVATE_ENTRY_STATUS, 'Private'),
        (DRAFT_ENTRY_STATUS, 'Draft'),
    )

    status = models.CharField(
        max_length=16,
        choices=ENTRY_STATUSES,
        default=PUBLIC_ENTRY_STATUS
    )

    objects = models.Manager()
    public = managers.PublicEntryManager()
    private = managers.PrivateEntryManager()
    draft = managers.DraftEntryManager()

    class Meta:
        verbose_name = 'Entry'
        verbose_name_plural = 'Entries'

        get_latest_by = 'created'
        ordering = ('-created',)

    def __str__(self):
        return self.title

    def __unicode__(self):
        return self.title

    def save(self, *args, **kwargs):
        self.teaser_html = render(self.teaser)
        self.body_html = render(self.body)

        super(Entry, self).save()

    @models.permalink
    def get_absolute_url(self):
        return ('djournal_entry_detail', [], {'slug': self.slug})



'''Feeds for Djournal.'''

from django.contrib.syndication.views import Feed
from django.core.urlresolvers import reverse
from django.shortcuts import get_object_or_404
from django.template import Context
from django.template.loader import get_template

from taggit.models import Tag

from djournal.models import Entry

class EntryFeed(Feed):

    title_template = 'djournal/feeds/entry_feed_item_title.html'
    description_template = 'djournal/feeds/entry_feed_item_description.html'

    def title(self):
        template = get_template('djournal/feeds/entry_feed_title.html')
        return template.render(Context({}))

    def description(self):
        template = get_template('djournal/feeds/entry_feed_description.html')
        return template.render(Context({}))

    def link(self):
        return reverse('djournal_entry_index')

    def items(self):
        return Entry.public.all()

    def item_title(self, item):
        return item.title

    def item_description(self, item):
        return item.teaser_html

    def item_link(self, item):
        return item.get_absolute_url()

class TaggedEntryFeed(Feed):

    title_template = 'djournal/feeds/tagged_entry_feed_item_title.html'
    description_template = 'djournal/feeds/tagged_entry_feed_item_description.html'

    def get_object(self, request, slug):
        return get_object_or_404(Tag, slug=slug)

    def items(self, obj):
        return Entry.public.filter(tags__in=[obj])

    def title(self, obj):
        template = get_template('djournal/feeds/tagged_entry_feed_title.html')
        return template.render(Context({'tag': obj}))

    def description(self, obj):
        template = get_template('djournal/feeds/tagged_entry_feed_description.html')
        return template.render(Context({'tag': obj}))

    def link(self, obj):
        return reverse('djournal_tagged_entry_index', kwargs={'slug': obj.slug})

    def item_title(self, item):
        return item.title

    def item_description(self, item):
        return item.teaser_html

    def item_link(self, item):
        return item.get_absolute_url()



'''URLs for Djournal.'''

from django.conf.urls.defaults import *

from djournal.feeds import EntryFeed, TaggedEntryFeed
from djournal.models import Entry
from djournal.sitemaps import EntrySitemap

urlpatterns = patterns('',
    url(r'^sitemap\.xml$', 'django.contrib.sitemaps.views.sitemap',
        {'sitemaps': {'entry': EntrySitemap}},
    ),
    url(r'^$', 'djournal.views.entry_index',
        name='djournal_entry_index',
    ),
    url(r'^feed/$', EntryFeed(),
        name='djournal_entry_feed'
    ),
    url(r'^tag/(?P<slug>.*)/feed/$', TaggedEntryFeed(),
        name='djournal_tagged_entry_feed',
    ),
    url(r'^tag/(?P<slug>.*)/$', 'djournal.views.tagged_entry_index',
        name='djournal_tagged_entry_index',
    ),
    url(r'^(?P<slug>.*)/$', 'djournal.views.entry_detail',
        name='djournal_entry_detail',
    ),
)



'''Managers for Djournal.'''

from django.db import models

class PublicEntryManager(models.Manager):
    def get_query_set(self):
        query_set = super(PublicEntryManager, self).get_query_set()
        return query_set.filter(status__exact=self.model.PUBLIC_ENTRY_STATUS)

class PrivateEntryManager(models.Manager):
    def get_query_set(self):
        query_set = super(PrivateEntryManager, self).get_query_set()
        return query_set.filter(status__exact=self.model.PRIVATE_ENTRY_STATUS)

class DraftEntryManager(models.Manager):
    def get_query_set(self):
        query_set = super(DraftEntryManager, self).get_query_set()
        return query_set.filter(status__exact=self.model.DRAFT_ENTRY_STATUS)



'''Admin specification for Djournal.'''

from datetime import datetime

from django.contrib import admin

from djournal.models import Entry

class EntryAdmin(admin.ModelAdmin):

    date_hierarchy = 'created'

    exclude = ('author', 'created', 'modified')

    fieldsets = (
        ('Meta', {
            'fields': ('title', 'subtitle', 'slug', 'status', 'tags'),
        }),
        ('Content', {
            'fields': ('teaser', 'body'),
        }),
    )

    list_display = (
        'title', 'subtitle', 'slug_link', 'author',
        'format_created', 'format_modified', 'status',
    )

    list_filter = (
        'author',
        'status',
    )

    prepopulated_fields = {
        'slug': ('title',),
    }

    search_fields = (
        'title', 'subtitle', 'slug', 'body', 'teaser',
    )

    def save_model(self, request, obj, form, change):
        if not change:
            obj.author = request.user
            obj.created = datetime.now()

        obj.modified = datetime.now()

        obj.save()

    def slug_link(self, obj):
        return '<a href="{url}">{slug}</a>'.format(
            url=obj.get_absolute_url(), 
            slug=obj.slug
        )

    slug_link.allow_tags = True

    def format_created(self, obj):
        return obj.created.strftime("%Y-%m-%d %H:%M")

    def format_modified(self, obj):
        return obj.modified.strftime("%Y-%m-%d %H:%M")

admin.site.register(Entry, EntryAdmin)



#! /usr/bin/env python

from distutils.core import setup

from djournal import __VERSION__

setup(
    name = 'djournal',
    version = __VERSION__,
    description = 'Djournal is a blog application for Django.',
    author = 'P.C. Shyamshankar',
    author_email = 'sykora@lucentbeing.com',

    packages = ['djournal', 'djournal.migrations'],

    url = 'http://github.com/sykora/djournal/',
    license = 'GNU General Public License v3.0',

    classifiers = (
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Programming Language :: Python',
    )
)



'''Sitemaps for Djournal.'''

from django.contrib.sitemaps import Sitemap

from djournal.models import Entry

class EntrySitemap(Sitemap):
    changefreq = 'daily'
    priority = 0.5

    def items(self):
        return Entry.public.all()

    def lastmod(self, obj):
        return obj.modified
